#!/usr/bin/python

from mod_utils              import *
from mod_ecu_screen         import *
from mod_ecu_default        import *
from mod_ecu_state          import *
from mod_ecu_parameter      import *
from mod_ecu_command        import *
from mod_ecu_identification import *
from mod_ecu_service        import *
from mod_ecu_mnemonic       import *
from mod_optfile            import *
from mod_ply				import *
from mod_elm                import ELM

import mod_globals

from xml.dom.minidom        import parse
import xml.dom.minidom
import struct
import sys
import os
import time

class ECU:
  '''Contain data for one specific ECU
     implement menu for ecu'''
  
  path = "../EcuRenault/Sessions/"
 
  getDTCmnemo     = ""
  resetDTCcommand = ""
  screens         = []
  Defaults        = {}
  Parameters      = {}
  States          = {}
  Identifications = {}
  Commands        = {}
  Services        = {}
  Mnemonics       = {}
  
  ecudata  = {}
  
  minimumrefreshrate = 0.200
    
  def __init__(self, cecu, tran, elm ):
    
    self.elm     = elm
    self.ecudata = cecu

    self.getDTCmnemo     = ""
    self.resetDTCcommand = ""
    self.screens         = []
    self.Defaults        = {}
    self.Parameters      = {}
    self.States          = {}
    self.Identifications = {}
    self.Commands        = {}
    self.Services        = {}
    self.Mnemonics       = {}
    
    print 'Deflen:',len(self.Defaults)

    print "ECU type: ", cecu['stdType']
 
    print "Loading PLY "
    self.calc = Calc()
    
    mdom = xml.dom.minidom.parse(self.path+self.ecudata['ModelId'])
    mdoc = mdom.documentElement
  
    print "Loading screens "
    self.screens = []
    sc_class = ecu_screens( self.screens, mdoc, tran )

    print "Loading optimyzer"
    self.defaults = []
    opt_file = optfile(self.path+self.ecudata['OptimizerId'])
       
    print "Loading defaults"
    df_class = ecu_defaults       ( self.Defaults,        mdoc, opt_file.dict, tran )
    print "Loading parameters"
    pr_class = ecu_parameters     ( self.Parameters,      mdoc, opt_file.dict, tran )
    print "Loading stattes"
    st_class = ecu_states         ( self.States,          mdoc, opt_file.dict, tran )
    print "Loading identifications"
    id_class = ecu_identifications( self.Identifications, mdoc, opt_file.dict, tran )
    print "Loading commands"
    cm_class = ecu_commands       ( self.Commands,        mdoc, opt_file.dict, tran )
    print "Loading services"
    sv_class = ecu_services       ( self.Services,        mdoc, opt_file.dict, tran )
    print "Loading mnemonics"
    mm_class = ecu_mnemonics      ( self.Mnemonics,       mdoc, opt_file.dict, tran )
    print "Loading DTC commands"
    self.getDTCmnemo, self.resetDTCcommand = df_class.getDTCCommands( mdoc, opt_file.dict, cecu['stdType'] )
  
    print "Init ELM"
    if self.ecudata['pin'].lower()=='can':
      self.elm.init_can()
      self.elm.set_can_addr( self.ecudata['dst'] )
      self.elm.start_session_can( self.ecudata['startDiagReq'] )
    else:
      self.elm.init_iso()
      self.elm.set_iso_addr( self.ecudata['dst'] )

    print "Done"  
    
  def show_commands(self, datarefs, path):
    while(True): 
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      print "Screen : "+path    
      menu = []
      cmds = []
      for dr in datarefs:
        datastr = dr.name;
        if dr.type=='State':
          datastr = self.States[dr.name].name + 'States not supported on one screen with commands'
        if dr.type=='Parameter':
          datastr = self.Parameters[dr.name].name + 'Parameters not supported on one screen with commands'
        if dr.type=='Identification':
          datastr = self.Identifications[dr.name].name + 'Identifications not supported on one screen with commands'
        if dr.type=='Command':
          datastr = dr.name + " [Command] " + self.Commands[dr.name].label
          cmds.append( dr.name )
        menu.append( datastr )

      menu.append("<Up>")
      choice = Choice(menu, "Choose :")
      if choice[0]=="<Up>": return
      
      header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'+ \
             "Screen : "+path+" -> "+cmds[int(choice[1])-1] + " [Command] "
      #executeCommand( self.Commands[cmds[int(choice[1])-1]], self.Services, self.elm, header )
        
  
  def show_datarefs(self, datarefs, path):
  
    #Check if datarefs contains any commands
    for dr in datarefs:
      if dr.type=='Command':
        self.show_commands(datarefs, path)
        return
    
    kb = KBHit()

    tb = time.time()   #time of beginning 

    if len(datarefs)==0 and 'DE' not in path: return
    
    page = 0
        
    while(True):
      
      strlst = []
      
      self.elm.clear_cache()
      
      for dr in datarefs:
        datastr = dr.name;
        help    = dr.type
        if dr.type=='State':
          datastr, help = get_state( self.States[dr.name], self.Mnemonics, self.elm, self.calc )
        if dr.type=='Parameter':
          datastr, help = get_parameter( self.Parameters[dr.name], self.Mnemonics, self.elm, self.calc )
        if dr.type=='Identification':
          datastr, help = get_identification( self.Identifications[dr.name], self.Mnemonics, self.elm, self.calc )       
        if dr.type=='Command':
          datastr = dr.name + " [Command] " + self.Commands[dr.name].label
        
        strlst.append( datastr )
        if mod_globals.opt_verbose and len(help)>0:
          tmp_str = ''
          for s in help:
            s = s.replace('\n',' ')
            s = s.replace('\r',' ')
            tmp_str = tmp_str + s + ' '
          i = 0
          W = 50
          while i*W<len(tmp_str):
            strlst.append('\t'+tmp_str[i*W:(i+1)*W])
            i=i+1
          strlst.append('')

      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      print "Screen : "+path
      
      H = 25
      pages = len(strlst)/H
      for l in strlst[page*H:(page+1)*H]:
        print l

      if pages>0:
        print ""
        print "[Page ",page+1," from ",pages+1,"] Press page number to switch or any other to exit"
      else:  
        print ""
        print "Press any key to exit"
      
      if kb.kbhit():
        c = kb.getch()
        n = ord(c)-ord('0')
        if n>0 and n<=(pages+1):
          page = n-1
          continue
        kb.set_normal_term()
        return
       
      # check refresh rate 
      tc = time.time()
      if ((tc-tb)<self.minimumrefreshrate):
        time.sleep(tb+self.minimumrefreshrate-tc)
      tb = tc
      
  def show_subfunction(self, subfunction, path):
    while(1): 
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      print "Screen : "+path+" -> "+subfunction.text    
      menu = []
      if len(subfunction.datarefs)!=0 and len(subfunction.datarefs)>0:
        self.show_datarefs(subfunction.datarefs, path+" -> "+subfunction.text )
        return
      else:
        return        
   
  def show_function(self, function, path):
    while(1): 
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      print "Screen : "+path+" -> "+function.text    
      menu = []
      if len(function.datarefs)!=0 and len(function.subfunctions)!=0:
        print "Warning: both datarefs and functions not empty"
      if len(function.subfunctions)!=0:
        for sfu in function.subfunctions:
          menu.append(sfu.text)
        menu.append("<Up>")
        choice = Choice(menu, "Choose :")
        if choice[0]=="<Up>": return
        self.show_subfunction(function.subfunctions[int(choice[1])-1], path+" -> "+function.text )    
      if len(function.datarefs)!=0:
        self.show_datarefs(function.datarefs, path+" -> "+function.text )
        return        
   
  def show_screen(self, screen):
    while(1): 
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      print "Screen : "+screen.name    
      menu = []
      if len(screen.datarefs)!=0 and len(screen.functions)!=0:
        print "Warning: both datarefs and functions not empty"
      if len(screen.functions)!=0:
        for fu in screen.functions:
          menu.append(fu.text)
        menu.append("<Up>")
        choice = Choice(menu, "Choose :")
        if choice[0]=="<Up>": return
        self.show_function(screen.functions[int(choice[1])-1],screen.name)    
      if len(screen.datarefs)!=0:
        self.show_datarefs(screen.datarefs, screen.name)
        return        
  
  def show_defaults_std_a(self):
    while(1):
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      
      path = "DE (STD_A)"
      print "Screen : " + path     
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
  
      dtcs, defstr, hlpstr = get_default_std_a( self.Defaults, self.Mnemonics, self.elm, self.calc, self.getDTCmnemo )
      
      for d in defstr.keys():
        menu.append(defstr[d])
         
      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'+ \
             "Screen : "+path
        executeCommand( self.Commands[self.resetDTCcommand], self.Services, self.elm, header )
        return
          
      dtchex = dtcs[int(choice[1])-1]
      
      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex[:4]].datarefs, path) 
    
  def show_defaults_std_b(self):
    while(1):
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      
      path = "DE (STD_B)"
      print "Screen : " + path     
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
  
      dtcs, defstr, hlpstr = get_default_std_b( self.Defaults, self.Mnemonics, self.elm, self.calc, self.getDTCmnemo )
      
      for d in defstr.keys():
        menu.append(defstr[d])
         
      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'+ \
             "Screen : "+path
        executeCommand( self.Commands[self.resetDTCcommand], self.Services, self.elm, header )
        return
          
      dtchex = dtcs[int(choice[1])-1]
      
      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex[:4]].datarefs, path) 
    

  def show_defaults_failflag(self):
    while(1):
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']
      
      path = "DE (FAILFLAG)"
      print "Screen : " + path     
      menu = []
      
      defstr = {}
      hlpstr = {}
  
      self.elm.clear_cache()
      
      dtcs, defstr, hlpstr = get_default_failflag( self.Defaults, self.Mnemonics, self.elm, self.calc )
      
      for d in sorted(defstr.keys()):
        menu.append(defstr[d])
         
      menu.append("<Up>")
      menu.append("<Clear>")
      choice = Choice(menu, "Choose one for detailed view or <Clear>:")
  
      if choice[0]=="<Up>": return
      if choice[0]=="<Clear>": 
        print "Executing command ", self.resetDTCcommand
        header = "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'+ \
             "Screen : "+path
        executeCommand( self.Commands[self.resetDTCcommand], self.Services, self.elm, header )
        return
          
      dtchex = dtcs[int(choice[1])-1]
      
      path = path+' -> '+defstr[dtchex]+'\n\n'+hlpstr[dtchex]+'\n'
      
      self.show_datarefs(self.Defaults[dtchex].datarefs, path) 
    
  def show_screens(self):
    while(1):
      os.system('cls' if os.name == 'nt' else 'clear')    # clear screen
      print "ECU : "+self.ecudata['ecuname']+'  '+self.ecudata['doc']+'\n'
      #print self.ecudata
      menu = []
      for l in self.screens:
        menu.append(l.name)
      menu.append("<Up>")
      choice = Choice(menu, "Choose :")
      if choice[0]=="<Up>": return
      
      if choice[0]=="DE": 
        if self.ecudata['stdType']=='STD_A':
          self.show_defaults_std_a()
        if self.ecudata['stdType']=='STD_B' or self.ecudata['stdType']=='UDS' :
          self.show_defaults_std_b()
        if self.ecudata['stdType']=='FAILFLAG':
          self.show_defaults_failflag()
        continue
          
      self.show_screen(self.screens[int(choice[1])-1])



    
    