#!/usr/bin/python

from mod_ecu_mnemonic   import *
from mod_utils          import Choice
from xml.dom.minidom    import parse
from xml.dom.minidom    import parseString
import xml.dom.minidom

def get_parameter( pr, mn, elm, calc ):
  comp = pr.computation
  comp = comp.replace("&amp;","&")
  comp = comp.strip()
  for m in sorted(pr.mnemolist, key=len, reverse=True):

    val = get_mnemonic( mn[m], elm )
    if mn[m].type=="SNUM8" and int(val,16)>0x7f:
      val = str(int(val,16)-0x100)
    elif mn[m].type=="SNUM16" and int(val,16)>0x7fff:
      val = str(int(val,16)-0x10000)
    elif mn[m].type=="SNUM32" and int(val,16)>0x7fffffff:
      val = str(int(val,16)-0x100000000)
    else:
      val = "0x"+val
          
    comp = comp.replace(m, val) 
    
  pr.value = calc.calculate(comp)
  if '.' in str(pr.value):
    pr.value = "%10.2f"%float(pr.value)
    
  tmpmin = str(pr.min)
  tmpmax = str(pr.max)
  if tmpmin.strip()=='0' and tmpmax.strip()=='0':
    tmpmin = ''
    tmpmax = ''
  
  return "%-5s %-50s %5s %10s %-10s %-5s"%(pr.name,pr.label,tmpmin,pr.value,pr.unit,tmpmax), pr.helps 

class ecu_parameter:

  name    = ""
  agcdRef = ""
  codeMR  = ""
  label   = ""
  unit    = ""
  type    = ""
  min     = 0
  value   = 0
  max     = 0
  format  = 0
  domains     = []
  helps       = []
  computation = ""
  mnemolist   = []
  
  def __init__(self, pr, opt, tran ):
    self.name = pr.getAttribute("name")
    self.agcdRef = pr.getAttribute("agcdRef")
    self.codeMR  = pr.getAttribute("codeMR")
    
    Label = pr.getElementsByTagName("Label")
    codetext = Label.item(0).getAttribute("codetext")
    if codetext: 
      self.label = tran[codetext]
    else:
      self.label = ""
      
    Unit = pr.getElementsByTagName("Unit")
    if Unit:
      codetext = Unit.item(0).getAttribute("codetext")
      if codetext: 
        self.unit = tran[codetext]
      else:
        self.label = ""
      
    format_tmp = pr.getElementsByTagName("Format")
    if format_tmp:
      format = format_tmp.item(0).firstChild.nodeValue
    
    Limits = pr.getElementsByTagName("Limits")
    if Limits:
      self.min = Limits.item(0).getAttribute("min")
      self.max = Limits.item(0).getAttribute("max")
      
    self.domains = []
    Domains = pr.getElementsByTagName("Domains")
    if Domains:
      for dm in Domains:
        Domain = dm.getElementsByTagName("Domain")
        if Domain:
          domain = Domain.item(0).firstChild.nodeValue
          self.domains.append(domain)
          
    self.helps = []
    Helps = pr.getElementsByTagName("Helps")
    if Helps:
      for hl in Helps:
        Lines = hl.getElementsByTagName("Line")
        if Lines:
          for ln in Lines:
            line = ""
            Label = ln.getElementsByTagName("Label")
            if Label:
              for la in Label:
                codetext = la.getAttribute("codetext")
                if codetext: 
                  line = line + tran[codetext]
            self.helps.append(line)
    
    xmlstr = opt["Parameter\\"+self.name]
    odom = xml.dom.minidom.parseString( xmlstr.encode( "utf-8" ) )
    odoc = odom.documentElement
    
    self.computation = ""
    Computation = odoc.getElementsByTagName("Computation")
    if Computation:
      for cmpt in Computation:
        self.type = cmpt.getAttribute("type")
        tmp = cmpt.getElementsByTagName("Value").item(0).firstChild.nodeValue
        self.computation = tmp.replace(" ","").replace("&amp;","&")
        
        self.mnemolist = []
        Mnemo = cmpt.getElementsByTagName("Mnemo")
        if Mnemo:
          for mn in Mnemo:
            self.mnemolist.append(mn.getAttribute("name"))
'''    
    print '='*60
    print self.name+" "+self.label+" "+self.min+" "+str(self.value)+self.unit+" "+self.max
    print "Format:"+str(self.format)
    print self.computation
    for l in self.mnemolist:
      print "\t"+l
    print '-'*60
    for l in self.helps:
      print l
    print '-'*60
'''


class ecu_parameters:
 
  def __init__(self, parameter_list, mdoc, opt, tran ):
    Parameters = mdoc.getElementsByTagName("Parameter")
    if Parameters:
      for pr in Parameters:
        parameter = ecu_parameter( pr, opt, tran )
        parameter_list[parameter.name] = parameter
        