#!/usr/bin/python

# allecus contain dictionary
#
#'dst': '7A'
#'src': 'F1'
#'OptimizerId': 'SG0110577.XML'
#'ModelId': 'FG0110577.XML'}
#'stdType': 'STD_A'
#'doc': 'DCM1.2_L4DB_58_ X61_A'
#'idf': '1'
#'pin': 'can'
#'pin1': '6'
#'pin2': '14'
#'startDiagReq': '10C0'
#'startDiagRsp': '50C0'
#'stopDiagReq': '1081'
#'stopDiagRsp': '5081'
#'ids': ['2180','6180','0','6','FF','58','2180','6180','0','16','FF','4D']

from xml.dom.minidom import parse
import xml.dom.minidom
import pickle


from mod_utils   import Choice
from mod_elm     import ELM
import mod_globals
import sys
import glob
import os

opt_demo = False

class ScanEcus:
  '''List all possible ECUs of car'''
  
  allecus = {};       #all ecus possible in this model
  detectedEcus = [];  #ecus detected in car during scan
  vhcls  = [];
  ecus   = [];
  models = [];
  reqres = [];        #results of previous requests
  selectedEcu = 0;
  
  def __init__(self, elm_ref):
    self.elm = elm_ref
    
    ####### Get list car models from vehicles directory #######
    self.vhcls  = [];
    for file in glob.glob("../Vehicles/TCOM_*.Xml"):
      DOMTree = xml.dom.minidom.parse(file)
      vh = DOMTree.documentElement
      if vh.hasAttribute("defaultText"):
        vehiclename = vh.getAttribute("defaultText")
        vehTypeCode = vh.getAttribute("vehTypeCode")
        self.vhcls.append([vehiclename,file,vehTypeCode])
          
  def scanAllEcus(self):
    '''scan all ecus'''
    
    if os.path.isfile("savedEcus.p") and not mod_globals.opt_scan: 
      self.detectedEcus = pickle.load( open( "savedEcus.p", "rb" ) )
      return
      
    self.reqres = []
    self.errres = []
    
    i = 0
    print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
    sys.stdout.flush()
    
    #scan CAN ecus
    self.elm.init_can()
    for ecu, row in sorted(self.allecus.iteritems(),key = lambda (x,y) :y['idf']):
      if (self.allecus[ecu]['pin']=='can' and
          self.allecus[ecu]['pin1']=='6'  and
          self.allecus[ecu]['pin2']=='14'):        
        i = i+1
        print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
        sys.stdout.flush()
        
        self.elm.set_can_addr( self.allecus[ecu]['dst'] )
        self.scan_can( self.allecus[ecu] )

    self.elm.close_protocol()

    #scan KWP ecud
    self.elm.init_iso()
    for ecu, row in sorted(self.allecus.iteritems(),key = lambda (x,y) :y['idf']):
      if (self.allecus[ecu]['pin']=='iso' and 
          self.allecus[ecu]['pin1']=='7'  and 
          self.allecus[ecu]['pin2']=='15'):
        
        i = i+1
        print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus)),
        sys.stdout.flush()
   
        self.elm.set_iso_addr( self.allecus[ecu]['dst'] )
        self.scan_iso( self.allecus[ecu] )
    print '\r'+'\t\t\t\t'+'\rScanning:'+str(i)+'/'+str(len(self.allecus))+" Detected: "+str(len(self.detectedEcus))
    
    #sort list of detected ECUs
    self.detectedEcus = sorted(self.detectedEcus, key=lambda k: int(k['idf']))
    pickle.dump( self.detectedEcus, open( "savedEcus.p", "wb" ) )
    #print self.detectedEcus
        
  def chooseECU(self, necu):
    
    if len(self.detectedEcus)==0:
      self.scanAllEcus()
    
    if len(self.detectedEcus)==0:
      print "NO ECU detected. Nothing to do. ((("
      exit( 2 )
    
    listecu = []
    
    print "\n    %-5s %-6s %-5s %-45s %s" % ("Addr","Family","Index","Name","Type")
    for row in self.detectedEcus:
      line = "%-4s %-6s %-5s %-45s %s" % (row['dst'], row['idf'],row['ecuname'],row['doc'],row['stdType'])
      listecu.append( line )
    listecu.append( "Exit" )    
    choice = Choice(listecu, "Choose ECU :")
    if choice[0]=="Exit": exit(1)
    i = int(choice[1])-1
    self.selectedEcu = i
    return self.detectedEcus[self.selectedEcu]
  
  def getselectedEcu(self):
    return self.detectedEcus[self.selectedEcu]
    
  def load_model_ECUs( self, file ):

    # loading name list of all possible ECUs
    
    ecuname = ''
    
    DOMTree = xml.dom.minidom.parse(file)
    vh = DOMTree.documentElement
    
    connector = vh.getElementsByTagName("Connector")
    cannetwork = connector.item(0).getElementsByTagName("CANNetwork")
    isonetwork = connector.item(0).getElementsByTagName("ISONetwork")
    
    for pin in cannetwork:
      canH = pin.getAttribute("canH")
      canL = pin.getAttribute("canL")
      eculist = pin.getElementsByTagName("EcuList")
      if eculist:
        ecukind = eculist.item(0).getElementsByTagName("EcuKind")
        for ek in ecukind:
          idf = ek.getAttribute("idFamily")
          ecuref = ek.getElementsByTagName("EcuRef")
          for er in ecuref:
            ecuname = er.getAttribute("name")
            ecudoc  = er.getAttribute("doc")
            self.allecus[ecuname] = {}
            self.allecus[ecuname]["pin"] ="can"
            self.allecus[ecuname]["pin1"]=canH
            self.allecus[ecuname]["pin2"]=canL
            self.allecus[ecuname]["idf"] =idf
            self.allecus[ecuname]["doc"] =ecudoc
            self.allecus[ecuname]["ecuname"] = ecuname
    
    for pin in isonetwork:
      pinK = pin.getAttribute("pinK")
      pinL = pin.getAttribute("pinL")
      eculist = pin.getElementsByTagName("EcuList")
      if eculist:
        ecukind = eculist.item(0).getElementsByTagName("EcuKind")
        for ek in ecukind:
          idf = ek.getAttribute("idFamily")
          ecuref = ek.getElementsByTagName("EcuRef")
          for er in ecuref:
            ecuname = er.getAttribute("name")
            ecudoc  = er.getAttribute("doc")
            self.allecus[ecuname] = {}
            self.allecus[ecuname]["pin"] ="iso"
            self.allecus[ecuname]["pin1"]=pinK
            self.allecus[ecuname]["pin2"]=pinL
            self.allecus[ecuname]["idf"] =idf
            self.allecus[ecuname]["doc"] =ecudoc
            self.allecus[ecuname]["ecuname"] = ecuname
        
    # Finding them in Uces.xml and loading 
    
    DOMTree = xml.dom.minidom.parse("../EcuRenault/Uces.Xml")
    Ecus = DOMTree.documentElement
    EcuDatas = Ecus.getElementsByTagName("EcuData")
    if EcuDatas:
      for EcuData in EcuDatas:
        name = EcuData.getAttribute("name")
        if name in self.allecus.keys():
          self.allecus[name]["stdType"] = EcuData.getAttribute("stdType")
          self.allecus[name]["ModelId"] = EcuData.getElementsByTagName("ModelId").item(0).firstChild.nodeValue
          self.allecus[name]["OptimizerId"] = EcuData.getElementsByTagName("OptimizerId").item(0).firstChild.nodeValue
          if self.allecus[name]["doc"]=='':
             self.allecus[name]["doc"] = self.allecus[name]["ModelId"]
          
          ecui = EcuData.getElementsByTagName("ECUInformations")
          if ecui:
            isodst = ""
            candst = ""
            src = "F1"
            
            fastini = ecui.item(0).getElementsByTagName("FastInitAddress")
            if fastini:
              isodst = fastini.item(0).getAttribute("value")
      
            addr = ecui.item(0).getElementsByTagName("Address")
            if addr:
              candst = addr.item(0).getAttribute("targetAddress")
              src = addr.item(0).getAttribute("toolAddress")
               
            self.allecus[name]["src"] = src
            if len(candst)>0:
              self.allecus[name]["dst"] = candst
            else:
              if len(isodst)>0: 
                self.allecus[name]["dst"] = isodst
              else:
                self.allecus[name]["dst"] = ''
               
            frms = ecui.item(0).getElementsByTagName("Frames")
            if frms:
              StartDiagSession = frms.item(0).getElementsByTagName("StartDiagSession")
              if StartDiagSession:
                self.allecus[name]["startDiagReq"] = StartDiagSession.item(0).getAttribute("request")
                self.allecus[name]["startDiagRsp"] = StartDiagSession.item(0).getAttribute("response")
              else:
                self.allecus[name]["startDiagReq"] = ''
                self.allecus[name]["startDiagRsp"] = ''
              StopDiagSession = frms.item(0).getElementsByTagName("StopDiagSession")
              if StopDiagSession:
                self.allecus[name]["stopDiagReq"] = StopDiagSession.item(0).getAttribute("request")
                self.allecus[name]["stopDiagRsp"] = StopDiagSession.item(0).getAttribute("response")
              else:
                self.allecus[name]["stopDiagReq"] = ''
                self.allecus[name]["stopDiagRsp"] = ''
            else:
              self.allecus[name]["startDiagReq"] = ''
              self.allecus[name]["startDiagRsp"] = ''
              self.allecus[name]["stopDiagReq"] = ''
              self.allecus[name]["stopDiagRsp"] = ''
            
            idtt = []
            ids  = ecui.item(0).getElementsByTagName("Ids")
            if ids:
              for id in ids:
                IdFrame = id.getElementsByTagName("IdFrame")
                if IdFrame:
                  idreq = IdFrame.item(0).getAttribute("request")
                  idrsp = IdFrame.item(0).getAttribute("response")
                  idlen = IdFrame.item(0).getAttribute("length")
                idbytes = id.getElementsByTagName("IdByte")
                if idbytes:
                  for idb in idbytes:
                    idrank = idb.getAttribute("rank")
                    idmask = idb.getAttribute("mask")
                    idval  = idb.getAttribute("value")
                    idtt.append(idreq)
                    idtt.append(idrsp)
                    idtt.append(idlen)
                    idtt.append(idrank)
                    idtt.append(idmask)
                    idtt.append(idval) 
            self.allecus[name]["ids"] = idtt  

  def chooseModel(self, num):
    
    for row in self.vhcls:
      self.models.append( row[2]+" "+row[0] )
        
    if num==0 or num>len(self.models):
      ch = Choice(self.models, "Choose model :")
    else:
      ch = [self.models[num-1],num]
    
    model = ch[0]
      
    print "Loading data for :", model, self.vhcls[int(ch[1])-1][1],
    sys.stdout.flush()
    
    self.allecus = {}
 
    self.load_model_ECUs( "../Vehicles/"+self.vhcls[int(ch[1])-1][1] )
    print "  - "+str(len(self.allecus))+" ecus loaded"
    
  def compare_ecu( self, row, rrsp, req ):
    
    base = 0
    res = 0
    att = 0
    
    while base+6<=len(row) and (int(row[base+3])*3+2)<len(rrsp):
      if row[base]!=req:
        req = row[base]
        rrsp = self.elm.cmd(req)

      byte = int(rrsp[int(row[base+3])*3:int(row[base+3])*3+2],16)
      mask = int(row[base+4],16)
      val  = int(row[base+5],16)
      
      if (byte&mask)==val:
        res += 1
      att += 1
      if att!=res:
        break
      base += 6
      
    if res==att and res>0:
      return True
    else:
      return False
       
  def request_can( self, row ):
   
    global opt_demo
    
    self.elm.start_session_can(row['startDiagReq']) 	#open session    
    rrsp = self.elm.cmd(row['ids'][0])	#get identification data
    
    rerr = ''
    if row['stdType']=='STD_A': 
      rerr = self.elm.cmd('17FF00')	    #get errors STD_A

    if row['stdType']=='STD_B': 
      rerr = self.elm.cmd('1902AF')	    #get errors STD_B
      
    if row['stdType']=='UDS': 
      rerr = self.elm.cmd('1902AF')	    #get errors UDS
    
    if len(row['stopDiagReq'])>0:
      self.elm.cmd(row['stopDiagReq'])	#close session
    
    res = ""
    for s in rrsp.split('\n'):
      if ':' in s:
        res += s[2:].strip()+' '
    rrsp = res
    res = ""
    for s in rerr.split('\n'):
      if ':' in s:
        res += s[2:].strip()+' '
      else:
        if ("57 " in s) or ("59 " in s) or ("7F " in s):
          res = s.strip()
    rerr = res
    return rrsp,rerr
  
  def request_iso( self, row ):
  
    global opt_demo
    
    if len(row['dst'])!=2: return 'addr error','addr error' 
    rsp = self.elm.cmd("81")	 		    #init bus
    rrsp = ""
    rerr = ""
    if "ERROR" not in rsp:
      self.elm.cmd(row['startDiagReq']) 	#open session
      rrsp = self.elm.cmd(row['ids'][0])	#get identification data

      rerr = ''
    
      if row['stdType']=='STD_A': 
        rerr = self.elm.cmd('17FF00')	    #get errors STD_A
  
      if row['stdType']=='STD_B': 
        rerr = self.elm.cmd('1902AF')	    #get errors STD_B
        
      if row['stdType']=='UDS': 
        rerr = self.elm.cmd('1902AF')	    #get errors UDS
      
      if len(row['stopDiagReq'])>0:
        self.elm.cmd(row['stopDiagReq'])	#close session
    
    res = ""
    for s in rrsp.split('\n'):
      if ' F1 ' in s:
        res += s[2:].strip()+' '
    rrsp = res
    res = ""
    for s in rerr.split('\n'):
      if ' F1 ' in s:
        res += s[2:].strip()+' '
    rerr = res
    return rrsp,rerr
    
  def scan_can( self, row ):
    
    rrsp = ''
    rerr = ''
    for r in self.reqres:
      if (row['dst']+row['startDiagReq']+row['stdType']+row['ids'][0])==r[0]:
        rrsp = r[1]
        rerr = r[2]
    if rrsp=='':
      rrsp,rerr = self.request_can( row )
      if not rrsp: rrsp = ''
      if not rerr: rerr = ''
      self.reqres.append([row['dst']+row['startDiagReq']+row['stdType']+row['ids'][0],rrsp,rerr]) #populate cache for not to ask again
  
    compres = False
    if 'ERROR' not in rrsp:
      rrsp = rrsp[3:]
      compres = self.compare_ecu( row['ids'], rrsp, row['ids'][0] )

    if compres:
      familynotdeteced = True
      for i in self.detectedEcus:
        if i['idf']==row['idf']:
          familynotdeteced = False
      if familynotdeteced:
        # than we found new ecu 
        row['rerr'] = rerr   
        self.detectedEcus.append(row)
    
  def scan_iso( self, row ):
    rrsp = ''
    rerr = ''
    for r in self.reqres:
      if (row['dst']+row['startDiagReq']+row['stdType']+row['ids'][0])==r[0]:
        rrsp = r[1]
        rerr = r[2]
    if rrsp=='':
      rrsp,rerr = self.request_iso( row )
      if not rrsp: rrsp = ''
      if not rerr: rerr = ''
      self.reqres.append([row['dst']+row['startDiagReq']+row['stdType']+row['ids'][0],rrsp,rerr]) #populate cache for not to ask again
    
    compres = False
    if 'ERROR' not in rrsp:
      rrsp = rrsp[3:]
      compres = self.compare_ecu( row['ids'], rrsp, row['ids'][0] )

    if not rerr: rerr = ''

    if compres:
      familynotdeteced = True
      for i in self.detectedEcus:
        if i[0]==row['idf']:
          familynotdeteced = False
      if familynotdeteced:    
        # than we found new ecu    
        self.detectedEcus.append(row)

if __name__ == "__main__":
  ce = ScanEcus( 0 )
  ce.chooseModel( 0 )
  ce.scanAllEcus()
  for l in ce.allecus:
    print ce.allecus[l]
