#!/usr/bin/python

import serial                                    #sudo easy_install pyserial
import sys, os

import mod_globals

from serial.tools  import list_ports
from mod_elm       import ELM
from mod_scan_ecus import ScanEcus
from mod_ecu       import ECU
from mod_optfile   import *


def optParser():
  '''Parsing of command line parameters. User should define at least com port name'''
  
  import optparse

  parser = optparse.OptionParser(
    usage = "%prog -p <port> [options]",
    version="%prog 0.8(beta)",
    description = "pyRen - python version of Reanult car diagnostic"
  )
  
  parser.add_option("-p",
      help="ELM327 com port name",
      dest="port",
      type="string")

  parser.add_option("-s",
      help="com port speed",
      dest="speed",
      default="38400",
      type="string")

  parser.add_option("-l",
      help="language option",
      dest="lang",
      default="RU")

  parser.add_option("-m",
      help="number of car model",
      dest="car",
      default=0,
      type="int")

  parser.add_option("-v",
      help="show parameter explanations",
      dest="verbose",
      default=False,
      action="store_true")

  parser.add_option("--log",
      help="log file name",
      dest="logfile",
      default="",
      type="string")

  parser.add_option("--demo",
      help="for debuging purpose. Work without car",
      dest="demo",
      default=False,
      action="store_true")

  parser.add_option("--scan",
      help="scan ECUs even if there is a savedEcus.p",
      dest="scan",
      default=False,
      action="store_true")


  (options, args) = parser.parse_args()
  
  if not options.port:
    parser.print_help()
    iterator = sorted(list(list_ports.comports()))
    print ""
    print "Available COM ports:"
    for port, desc, hwid in iterator:
      print "%-30s \n\tdesc: %s \n\thwid: %s" % (port,desc,hwid)
    print ""
    exit(2)
  else:
    mod_globals.opt_port    = options.port
    mod_globals.opt_speed   = options.speed
    mod_globals.opt_lang    = options.lang
    mod_globals.opt_car     = options.car
    mod_globals.opt_log     = options.logfile
    mod_globals.opt_demo    = options.demo
    mod_globals.opt_scan    = options.scan
    mod_globals.opt_verbose = options.verbose

def main():
  '''Main function'''

  optParser()
  
  print 'Opening ELM'
  elm = ELM( mod_globals.opt_port, int(mod_globals.opt_speed), mod_globals.opt_log )

  print 'Loading ECUs list'
  se  = ScanEcus(elm)                    #Prepare list of all ecus

  if not os.path.isfile("savedEcus.p") or mod_globals.opt_scan: 
    # choosing model 
    se.chooseModel( mod_globals.opt_car )  #choose model of car for doing full scan
    print 'Do scanning'
    se.scanAllEcus()                       #First scan of all ecus
  
  print "Loading language ",
  sys.stdout.flush()
                                         #loading language data
  lang = optfile("../Location/DiagOnCan_"+mod_globals.opt_lang+".bqm")
  print "Done"

  while( 1 ):
    os.system('cls' if os.name == 'nt' else 'clear')      # clear screen
    choosen_ecu = se.chooseECU( mod_globals.opt_ecu )     # choose ECU among detected
    ecu = ECU(choosen_ecu, lang.dict, elm )               # loading data for chosen ECU
    
    ecu.show_screens()                                    # show ECU screens

if __name__ == '__main__':
  main()
  